import random
import time
from abc import ABC
from typing import Optional
from lgt_jobs.lgt_common.discord_client.discord_client import DiscordClient
from lgt_jobs.lgt_common.helpers import get_formatted_bot_name
from lgt_jobs.lgt_common.slack_client.web_client import SlackWebClient
from lgt_jobs.lgt_data.enums import SourceType
from lgt_jobs.lgt_data.model import DedicatedBotModel, Server, Channel, DiscordUser
from lgt_jobs.lgt_data.mongo_repository import DedicatedBotRepository, UserMongoRepository
from pydantic import BaseModel
from lgt_jobs.lgt_data.analytics import get_bots_aggregated_analytics
import logging as log
from lgt_jobs.lgt_common.enums.slack_errors import SlackErrors
from lgt_jobs.basejobs import BaseBackgroundJob, BaseBackgroundJobData

"""
Update bots statistics
"""


class BotStatsUpdateJobData(BaseBackgroundJobData, BaseModel):
    bot_id: Optional[str]


class BotStatsUpdateJob(BaseBackgroundJob, ABC):
    @property
    def job_data_type(self) -> type:
        return BotStatsUpdateJobData

    def exec(self, data: BotStatsUpdateJobData):
        bots_rep = DedicatedBotRepository()
        users_rep = UserMongoRepository()
        bot = bots_rep.get_one(id=data.bot_id, include_deleted=True)

        if bot.source.source_type == SourceType.DISCORD:
            client = DiscordClient(bot.token)
            servers_response = client.get_servers()

            if isinstance(servers_response, dict):
                if servers_response.get('message') == '401: Unauthorized':
                    BotStatsUpdateJob.__updated_invalid_creds_flag(bot, True)
                if servers_response.get('code') == 40002:
                    bot.two_factor_required = True
                    bots_rep.add_or_update(bot)
                log.warning(f"[BotStatsUpdateJob]: Servers response is not list: {servers_response}")
                return

            discord_servers = [Server.from_dic(server) for server in servers_response]
            user = users_rep.get(bot.user_id)
            discord_user: DiscordUser = user.get_discord_user(bot.user_name)
            available_channels_types = [0, 5]
            if discord_user:
                discord_user.workspaces = discord_servers
                users_rep.set(user.id, discord_users=[user.to_dic() for user in user.discord_users])
            for discord_server in discord_servers:
                server = next(filter(lambda x: x.id == discord_server.id, bot.servers), None)
                if server:
                    discord_server.active = server.active
                    discord_server.deleted = server.deleted
                else:
                    discord_server.deleted = True
                discord_channels = [Channel.from_dic(channel) for channel in client.get_channels(discord_server.id)
                                    if channel.get('type', 0) in available_channels_types]
                for discord_channel in discord_channels:
                    if server:
                        channel = next(filter(lambda x: x.id == discord_channel.id, server.channels), None)
                        if channel:
                            discord_channel.active = channel.active
                discord_server.channels = discord_channels
                if discord_server.icon:
                    discord_server.icon = (f'https://cdn.discordapp.com/icons/{discord_server.id}/'
                                           f'{discord_server.icon}.png')
                time.sleep(random.randint(1, 2))

            bot.servers = discord_servers
            bot.two_factor_required = False
            bot.invalid_creds = False
            bots_rep.add_or_update(bot)
            return

        client = SlackWebClient(bot.token, bot.cookies)
        test_auth_response = client.test_auth()
        if not test_auth_response.status_code == 200:
            log.warning(f"[BotStatsUpdateJob]: Error to auth {data.bot_id}. {test_auth_response.content}")
            return

        if not bot.invalid_creds:
            error = test_auth_response.json().get("error")
            if error == SlackErrors.INVALID_AUTH or error == SlackErrors.TWO_FACTOR_REQUIRED:
                BotStatsUpdateJob.__updated_invalid_creds_flag(bot, True)

        user = test_auth_response.json().get('user_id')
        bot.associated_user = user
        bots_rep.add_or_update(bot)
        team_info = {}
        try:
            team_info = client.get_team_info()
        except:
            log.warning(f"[BotStatsUpdateJob]: Error to get  team info of {data.bot_id}. {test_auth_response.content}")

        if team_info.get('ok'):
            bot_name = get_formatted_bot_name(team_info['team']['domain'])
            if bot.servers:
                bot.servers[0].name = bot_name
            bot.source.source_name = bot_name
            bot.slack_url = bot.registration_link = team_info['team']['url']
            bots_rep.add_or_update(bot)
        received_messages, filtered_messages = get_bots_aggregated_analytics(bot_ids=[bot.id])
        try:
            channels_response = client.channels_list()
        except:
            log.warning(f"[BotStatsUpdateJob]: Error to get channels list for bot {bot.id}.")
            return

        if not channels_response['ok']:
            if channels_response.get("error") == SlackErrors.INVALID_AUTH:
                BotStatsUpdateJob.__updated_invalid_creds_flag(bot, True)
            else:
                log.warning(f"[BotStatsUpdateJob]: Error during update bot {bot.id} stats. Error: {channels_response}")
            return
        channels = channels_response['channels']
        connected_channels = 0
        channels_users = {}
        active_channels = {}
        users_count = 0
        for channel in channels:
            if channel['is_member']:
                active_channels[channel['id']] = channel['name']
                connected_channels += 1
            num_members = channel.get('num_members', 0)
            channels_users[channel['id']] = num_members
            users_count += num_members

        bot.active_channels = active_channels
        bot.messages_received = received_messages.get(bot.source.source_id, 0)
        bot.messages_filtered = filtered_messages.get(bot.source.source_id, 0)
        bot.connected_channels = connected_channels
        bot.channels = len(channels)
        bot.channels_users = channels_users
        if bot.recent_messages is None:
            bot.recent_messages = []
        if bot.servers:
            channels = [Channel.from_dic(channel_dict) for channel_dict in channels]
            for channel in channels:
                slack_channel: Channel = next(filter(lambda x: x.id == channel.id, bot.servers[0].channels), None)
                if slack_channel:
                    channel.active = slack_channel.active
                    channel.subscribers = channels_users.get(slack_channel.id, 0)
                else:
                    channel.active = channel.is_member
            bot.servers[0].channels = channels
            bot.servers[0].subscribers = users_count

        # save only last 50 messages
        bot.recent_messages = bot.recent_messages[-50:]
        bots_rep.add_or_update(bot)

    @staticmethod
    def __updated_invalid_creds_flag(bot: DedicatedBotModel, invalid_creds: bool):
        if bot.invalid_creds != invalid_creds:
            bot.invalid_creds = invalid_creds
            DedicatedBotRepository().add_or_update(bot)



from abc import ABC
from typing import Dict
import requests
from lgt_jobs.lgt_common.helpers import update_credentials, get_formatted_bot_name
from lgt_jobs.lgt_common.slack_client.web_client import SlackWebClient
from lgt_jobs.lgt_data.enums import StatusConnection, SourceType
from lgt_jobs.lgt_data.model import UserWorkspace, SlackUser, DedicatedBotModel
from lgt_jobs.lgt_data.mongo_repository import UserMongoRepository, DedicatedBotRepository
from lgt_jobs.basejobs import BaseBackgroundJobData, BaseBackgroundJob
from .bot_stats_update import BotStatsUpdateJob, BotStatsUpdateJobData
from lgt_jobs.runner import BackgroundJobRunner
import logging as log
from pydantic import BaseModel

"""
Connect Slack Account
"""


class ConnectSlackAccountJobData(BaseBackgroundJobData, BaseModel):
    slack_email: str
    current_user_email: str
    code: str
    user_agent: str


class ConnectSlackAccountJob(BaseBackgroundJob, ABC):
    @property
    def job_data_type(self) -> type:
        return ConnectSlackAccountJobData

    def exec(self, data: ConnectSlackAccountJobData):
        current_user = UserMongoRepository().get_by_email(data.current_user_email)
        slack_user = current_user.get_slack_user(data.slack_email)
        if not slack_user:
            slack_user = SlackUser()
            slack_user.email = data.slack_email
            current_user.slack_users.append(slack_user)
        slack_user.status = StatusConnection.IN_PROGRESS

        client = SlackWebClient('')
        code_confirmed_response = client.confirm_code(data.slack_email, data.code, data.user_agent)
        if code_confirmed_response.status_code != 200:
            log.warning(f'Unable to confirm code due to error: {code_confirmed_response.content}')
            slack_user.status = StatusConnection.FAILED
            UserMongoRepository().set(current_user.id,
                                      slack_users=[user.to_dic() for user in current_user.slack_users])
            return

        code_confirmed = code_confirmed_response.json().get('ok', False)
        if not code_confirmed:
            slack_user.status = StatusConnection.FAILED
            UserMongoRepository().set(current_user.id,
                                      slack_users=[user.to_dic() for user in current_user.slack_users])
            log.warning(f'Invalid code')
            return

        slack_user.cookies = client.client.cookies = code_confirmed_response.cookies.get_dict()

        workspaces_response = client.find_workspaces(data.user_agent)
        if workspaces_response.status_code != 200:
            slack_user.status = StatusConnection.FAILED
            UserMongoRepository().set(current_user.id,
                                      slack_users=[user.to_dic() for user in current_user.slack_users])
            log.warning(f'Unable to get workspaces due to error: {workspaces_response.content}')
            return
        if not workspaces_response.json().get('ok', False):
            slack_user.status = StatusConnection.FAILED
            log.warning(f'Unable to get workspaces due to error: {workspaces_response.content}')
            return

        UserMongoRepository().set(current_user.id,
                                  slack_users=[user.to_dic() for user in current_user.slack_users])

        log.info(f'{slack_user.email}: got workspaces data {workspaces_response.json()}')
        user_workspaces = next((user for user in workspaces_response.json()['current_teams']
                                if user['email'] == data.slack_email), {}).get('teams', [])
        user_workspaces = [UserWorkspace.from_dic(ws) for ws in user_workspaces]
        user_workspaces = sorted(user_workspaces, key=lambda ws: ws.domain)

        session = requests.Session()
        session.cookies = code_confirmed_response.cookies
        session.headers.update({'User-Agent': data.user_agent})
        log.info(f'{slack_user.email}: started getting of tokens for {len(user_workspaces)} workspaces')
        for workspace in user_workspaces:
            if not workspace.magic_login_code:
                continue
            login_url = f"https://app.slack.com/t/{workspace.domain}/login/{workspace.magic_login_code}"
            magic_response = session.post(login_url, cookies=session.cookies, headers=session.headers)
            content = magic_response.content.decode('utf-8')
            start_token_index = content.find("xox")
            sliced_content = content[start_token_index:]
            end_token_index = sliced_content.find('"')
            token = sliced_content[:end_token_index]
            workspace.magic_login_url = login_url
            workspace.token = token
            workspace.domain = get_formatted_bot_name(workspace.domain)

        slack_user.cookies = session.cookies.get_dict()
        slack_user.workspaces = user_workspaces
        slack_user.status = StatusConnection.COMPLETE

        slack_users_dict = [user.to_dic() for user in current_user.slack_users]
        UserMongoRepository().set(current_user.id,
                                  slack_users=slack_users_dict)

        dedicated_bots_repository = DedicatedBotRepository()
        dedicated_bots = dedicated_bots_repository.get_all(user_id=current_user.id, user_name=data.slack_email,
                                                           source_type=SourceType.SLACK, include_deleted=True)
        user_workspaces_map: Dict[str, UserWorkspace] = {workspace.id: workspace
                                                         for workspace in user_workspaces if workspace.token}
        for workspace_id, workspace in user_workspaces_map.items():
            dedicated_bot = next(filter(lambda x:
                                        x.source.source_id == workspace_id and x.user_name == slack_user.email,
                                        dedicated_bots), None)
            if dedicated_bot:
                dedicated_bot: DedicatedBotModel = update_credentials(dedicated_bot, workspace.token,
                                                                      slack_user.cookies)
                dedicated_bot.two_factor_required = workspace.two_factor_required
                dedicated_bot.banned = False
                dedicated_bot.source.source_name = workspace.domain
                dedicated_bot.registration_link = dedicated_bot.slack_url = workspace.url
                dedicated_bots_repository.add_or_update(dedicated_bot)
                BackgroundJobRunner.submit(BotStatsUpdateJob, BotStatsUpdateJobData(bot_id=str(dedicated_bot.id)))

        for dedicated_bot in dedicated_bots:
            client = SlackWebClient(dedicated_bot.token, dedicated_bot.cookies)
            auth = client.test_auth()
            if auth.status_code == 200 and dedicated_bot.source.source_id not in user_workspaces_map.keys():
                dedicated_bot.banned = True
                dedicated_bot.invalid_creds = True
                for server in dedicated_bot.servers:
                    server.deleted = True
                dedicated_bots_repository.add_or_update(dedicated_bot)



from abc import ABC
import logging as log
from lgt_jobs.lgt_common.slack_client.web_client import SlackWebClient
from lgt_jobs.lgt_data.enums import DefaultBoards
from lgt_jobs.lgt_data.model import UserModel, DedicatedBotModel, SlackMemberInformation, UserLeadModel
from lgt_jobs.lgt_data.mongo_repository import UserMongoRepository, DedicatedBotRepository, \
    SlackContactUserRepository, BoardsMongoRepository, UserContactsRepository
from pydantic import BaseModel
from lgt_jobs.basejobs import BaseBackgroundJob, BaseBackgroundJobData
from lgt_jobs.services.web_client import V3ServerClient
from requests import exceptions

"""
Save inbox leads
"""


class InboxLeadsJobData(BaseBackgroundJobData, BaseModel):
    pass


class InboxLeadsJob(BaseBackgroundJob, ABC):
    v3_client: V3ServerClient

    @property
    def job_data_type(self) -> type:
        return InboxLeadsJobData

    def exec(self, _: InboxLeadsJobData):
        self.v3_client = V3ServerClient()
        users = UserMongoRepository().get_users()
        for user in users:
            log.info(f'[InboxLeadsJob]: Loading chat for the: {user.email}')
            dedicated_bots = DedicatedBotRepository().get_all(only_valid=True, user_id=user.id)
            for dedicated_bot in dedicated_bots:
                self.create_inbox_leads(user, dedicated_bot)

    def create_inbox_leads(self, user: UserModel, dedicated_bot: DedicatedBotModel):
        inbox_board = BoardsMongoRepository().get(user.id, is_primary=True, name=DefaultBoards.Inbox.value)
        if not inbox_board:
            return
        slack_client = SlackWebClient(dedicated_bot.token, dedicated_bot.cookies)

        attempt = 0
        conversations_list = []
        while attempt < 3:
            try:
                conversations_list = slack_client.get_im_list().get('channels', [])
                break
            except exceptions.JSONDecodeError as er:
                log.info(f'[InboxLeadsJob]: Loading chat failed for the: {dedicated_bot.id}. '
                         f'Attempt: {attempt}. {str(er)}')
                attempt += 1

        log.info(f'[InboxLeadsJob]: Loading chat for the: {dedicated_bot.id}. '
                 f'Count of chats: {len(conversations_list)}')
        for conversation in conversations_list:
            sender_id = conversation.get('user')
            im_id = conversation.get('id')
            if sender_id == "USLACKBOT":
                continue

            history = {}
            while attempt < 3:
                try:
                    history = slack_client.chat_history(im_id)
                    break
                except exceptions.JSONDecodeError as er:
                    log.info(f'[InboxLeadsJob]: Loading chat failed for the: {dedicated_bot.id}. '
                             f'Attempt: {attempt}. {str(er)}')
                    attempt += 1

            if not history.get('ok', False):
                log.warning(f'Failed to load chat for the: {dedicated_bot.id}. ERROR: {history.get("error", "")}')
                continue

            messages = history.get('messages', [])
            log.info(f'[InboxLeadsJob]: Count of messages: {len(messages)} with {sender_id}')
            if messages:
                user_contact = UserContactsRepository().find_one(user.id, sender_id=sender_id)
                log.info(f"[InboxLeadsJob]: user contact exists: {str(user.id)}:{sender_id}")
                if not user_contact:
                    people = SlackContactUserRepository().find_one(sender_id)
                    if not people:
                        slack_profile = slack_client.get_profile(sender_id).get('user')
                        InboxLeadsJob.create_people(slack_profile, dedicated_bot)

                    save_lead_response = self.v3_client.save_lead_from_contact(sender_id=sender_id,
                                                                               source_id=dedicated_bot.source.source_id,
                                                                               email=user.email,
                                                                               message=messages[0].get("text"))
                    log.info(f"[InboxLeadsJob]: Save lead for user: {user.email}")
                    if save_lead_response.status_code == 200:
                        lead = UserLeadModel.from_dic(save_lead_response.json())
                        self.v3_client.update_user_lead(lead.message.message_id, user.email,
                                                        im_id, str(inbox_board[0].id), "Received")
                        log.info(f"[InboxLeadsJob]: Added inbox lead {lead.id} for user: {user.email}")

                    else:
                        log.warning(f"[InboxLeadsJob]: Error to save lead from contact. "
                                    f"Details {save_lead_response.json()}")

    @staticmethod
    def create_people(slack_profile: dict, dedicated_bot: DedicatedBotModel):
        member_info: SlackMemberInformation = SlackMemberInformation.from_slack_response(slack_profile,
                                                                                         dedicated_bot.source)
        SlackContactUserRepository().collection().update_one({"sender_id": member_info.sender_id,
                                                              "source.source_id": dedicated_bot.source.source_id},
                                                             {"$set": member_info.to_dic()}, upsert=True)
        return SlackContactUserRepository().find_one(member_info.sender_id)



import json
from urllib import request
from pkg_resources import parse_version
from setuptools import setup, find_packages

package_name = "leadguru_jobs"


def versions():
    url = f'https://pypi.python.org/pypi/{package_name}/json'
    releases = json.loads(request.urlopen(url).read())['releases']
    return sorted(releases, key=parse_version, reverse=True)


try:
    version_parts = versions()[0].split(".")
except:
    version_parts = "0.0.0".split(".")

version_parts[1] = f'{float(version_parts[1]) + 1}'
last_version = ".".join(version_parts[0:-1])

install_requires = [
    'cachetools==3.1.0',
    'pydantic',
    'wheel',
    'setuptools',
    'twine',
    'build',
    'kubernetes',
    'pyyaml',
    'pymongo',
    'pytz',
    'requests',
    'mongoengine',
    'google',
    'google-cloud-pubsub',
    'google-cloud-storage',
    'websockets',
    'aiohttp',
    'nameparser',
    'loguru',
    'redmail'
]

setup(name=package_name,
      version=f'{last_version}',
      description='LGT jobs builds',
      packages=find_packages(include=['lgt_jobs', 'lgt_jobs.*']),
      package_data={'lgt_jobs': ['templates/new_message.html', 'assets/images/arrow.png', 'assets/images/firework.png',
                                 'assets/images/lock.png', 'assets/images/logo.png', 'assets/images/mail.png']},
      include_package_data=True,
      install_requires=install_requires,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3 :: Only',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: 3.9',
          'Intended Audience :: Developers',
          'Intended Audience :: Information Technology',
          'Intended Audience :: System Administrators',
          'Environment :: Console',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Internet',
      ],
      author_email='developer@leadguru.co',
      zip_safe=False)



from __future__ import annotations
import os
from collections import OrderedDict
from datetime import datetime, UTC
from bson import ObjectId
from mongoengine import connect, Document, DateTimeField, StringField, IntField, ObjectIdField, ListField
from typing import Dict, Tuple, Optional
from .mongo_repository import to_object_id, UserMongoRepository

connect(host=os.environ.get('MONGO_CONNECTION_STRING', 'mongodb://127.0.0.1:27017/'), db="lgt_admin", alias="lgt_admin")


class GlobalUserConfiguration(Document):
    created_at = DateTimeField(required=True)
    updated_at = DateTimeField(required=True)
    created_by = ObjectIdField(required=False)
    updated_by = ObjectIdField(required=False)
    dedicated_bots_days_to_remove = IntField(required=True)

    meta = {"db_alias": "lgt_admin"}

    @staticmethod
    def get_config() -> GlobalUserConfiguration:
        items = list(GlobalUserConfiguration.objects())
        if not items:
            # create default config
            GlobalUserConfiguration(
                created_at=datetime.now(UTC),
                updated_at=datetime.now(UTC),
                dedicated_bots_days_to_remove=10
            ).save()
            items = list(GlobalUserConfiguration.objects())
        return items[-1]


class UserTrackAction(Document):
    user_id = ObjectIdField(required=True)
    action = StringField(required=True)
    metadata = StringField(required=False)
    created_at = DateTimeField(required=True)
    meta = {"indexes": ["user_id"], "db_alias": "lgt_admin"}

    @staticmethod
    def get_aggregated(from_date: datetime = None, to_date: datetime = None) \
            -> Dict[str, Tuple[datetime, datetime]]:
        pipeline = [
            {
                "$group": {
                    "_id": "$user_id",
                    "last_action_at": {"$max": "$created_at"},
                    "first_action_at": {"$min": "$created_at"}
                }
            }]

        if from_date:
            beginning_of_the_day = datetime(from_date.year, from_date.month, from_date.day, 0, 0, 0, 0)
            pipeline.insert(0, {"$match": {"created_at": {"$gte": beginning_of_the_day}}})

        if to_date:
            end_of_the_day = datetime(to_date.year, to_date.month, to_date.day, 23, 59, 59, 999)
            pipeline.insert(0, {"$match": {"created_at": {"$lte": end_of_the_day}}})

        result = list(UserTrackAction.objects.aggregate(*pipeline))

        return {str(item.get("_id")): (item["first_action_at"], item["last_action_at"]) for item in result}

    @staticmethod
    def track(user_id: str | ObjectId, action: str, metadata: Optional[str] = None):
        UserTrackAction(
            user_id=to_object_id(user_id),
            created_at=datetime.now(UTC),
            action=action,
            metadata=metadata
        ).save()

    @staticmethod
    def get_ws_monitoring_logs(email: str, action_subtype: str, from_date: datetime, to_date: datetime,
                               source_id=None) -> list[UserTrackAction]:

        pipeline = {'action': {'$regex': f"^monitoring.*{action_subtype}$"}}

        if email:
            user = UserMongoRepository().get_by_email(email)
            pipeline['user_id'] = to_object_id(user.id)

        if from_date:
            start = datetime(from_date.year, from_date.month, from_date.day)
            pipeline['created_at__gte'] = start

        if to_date:
            end = datetime(to_date.year, to_date.month, to_date.day, 23, 59, 59)
            pipeline['created_at__lte'] = end

        if source_id:
            pipeline['metadata'] = {'$regex': f"^{source_id}:"}

        return list(UserTrackAction.objects(**pipeline))

    @staticmethod
    def get_global_user_actions(user_id: str = None, from_date: datetime = None,
                                to_date: datetime = None, actions: list = None) -> \
            Dict[str, Dict[str, Dict[str, int]]]:
        pipeline = [
            {
                '$addFields': {
                    'created_at_formatted': {
                        '$dateToString': {
                            'format': '%Y-%m-%d',
                            'date': '$created_at'
                        }
                    }
                }
            }, {
                '$group': {
                    '_id': '$created_at_formatted',
                    'count': {
                        '$sum': 1
                    }
                }
            }
        ]

        if actions:
            pipeline.insert(0, {'$match': {'action': {'$in': actions}}})

        if user_id:
            pipeline.insert(0, {"$match": {"user_id": to_object_id(user_id)}})

        if from_date:
            beginning_of_the_day = datetime(from_date.year, from_date.month, from_date.day, 0, 0, 0, 0)
            pipeline.insert(0, {"$match": {"created_at": {"$gte": beginning_of_the_day}}})

        if to_date:
            end_of_the_day = datetime(to_date.year, to_date.month, to_date.day, 23, 59, 59, 999)
            pipeline.insert(0, {"$match": {"created_at": {"$lte": end_of_the_day}}})

        analytics = list(UserTrackAction.objects.aggregate(*pipeline))
        analytics_dic = OrderedDict()
        for item in analytics:
            analytics_dic[item["_id"]] = item["count"]

        return analytics_dic

    @staticmethod
    def get_daily_user_actions(user_id: str = None, from_date: datetime = None,
                               to_date: datetime = None, actions: list = None) -> \
            Dict[str, Dict[str, Dict[str, int]]]:
        pipeline = [
            {
                '$match': {
                    '$and': [
                        {'action': {'$ne': 'login'}},
                        {'action': {'$ne': 'chat.message'}}
                    ]
                }
            },
            {
                '$group': {
                    '_id': {
                        'date': {
                            '$dateFromParts': {
                                'day': {
                                    '$dayOfMonth': '$created_at'
                                },
                                'month': {
                                    '$month': '$created_at'
                                },
                                'year': {
                                    '$year': '$created_at'
                                }
                            }
                        },
                        'action': '$action',
                        'user_id': '$user_id'
                    },
                    'count': {
                            '$sum': 1
                    }
                }
            }
        ]

        if actions:
            pipeline.insert(0, {'$match': {'action': {'$in': actions}}})

        if user_id:
            pipeline.insert(0, {"$match": {"user_id": to_object_id(user_id)}})

        if from_date:
            beginning_of_the_day = datetime(from_date.year, from_date.month, from_date.day, 0, 0, 0, 0)
            pipeline.insert(0, {"$match": {"created_at": {"$gte": beginning_of_the_day}}})

        if to_date:
            end_of_the_day = datetime(to_date.year, to_date.month, to_date.day, 23, 59, 59, 999)
            pipeline.insert(0, {"$match": {"created_at": {"$lte": end_of_the_day}}})

        analytics = list(UserTrackAction.objects.aggregate(*pipeline))

        result = {}
        for item in analytics:
            date = item["_id"]["date"].strftime('%Y-%m-%d')
            user_id = str(item["_id"]["user_id"])
            if date not in result:
                result[date] = {}
            if user_id not in result[date]:
                result[date][user_id] = {}
            result[date][user_id][item["_id"]["action"]] = item["count"]

        return result


class DelayedJob(Document):
    created_at = DateTimeField(required=True)
    scheduled_at = DateTimeField(required=True)
    job_type = StringField(required=True)
    data = StringField(required=True)
    jib = StringField(required=True)
    executed_at: DateTimeField(required=False)

    meta = {"indexes": ["-scheduled_at", "jib"], "db_alias": "lgt_admin"}


class UserCreditStatementDocument(Document):
    meta = {"indexes": [("user_id", "created_at"),
                        ("user_id", "created_at", "action")], "db_alias": "lgt_admin"}

    user_id = ObjectIdField(required=True)
    created_at = DateTimeField(required=True)
    balance = IntField(required=True)
    action = StringField(required=True)
    lead_id = StringField(required=False)
    attributes = ListField(field=StringField(), required=False)




import requests
from google.cloud import storage
from datetime import datetime, timedelta
from lgt_jobs.lgt_data.model import ChatMessage
from .slack_client import SlackClient
from ...lgt_data.mongo_repository import to_object_id


def get_file_url(blob_path):
    storage_client = storage.Client()
    bucket = storage_client.get_bucket(SlackFilesClient.bucket_name)
    blob = bucket.get_blob(blob_path)
    if not blob:
        return None
    # valid for 3 days
    return blob.generate_signed_url(timedelta(3))


class SlackMessageConvertService:
    @staticmethod
    def from_slack_response(user_email, bot_name, bot_token, dic, sender_id, bot_id, user_id, cookies=None):

        """
        :rtype: SlackHistoryMessageModel
        """
        result = ChatMessage()
        result.sender_id = sender_id
        result.bot_id = bot_id
        result.text = dic.get('text', '')
        result.user = dic.get('user', '')
        result.id = dic.get('ts', '')
        result.attachments = dic.get('attachments', [])
        result.files = []
        result.user_id = to_object_id(user_id)

        if 'files' in dic:
            for file in dic.get('files'):
                if file.get('mode', '') == "tombstone" or not file.get('url_private_download'):
                    continue
                new_file = ChatMessage.SlackFileModel()
                new_file.id = file.get('id')
                new_file.name = file.get('name')
                new_file.title = file.get('title')
                new_file.filetype = file.get('filetype')
                new_file.size = file.get('size')
                new_file.mimetype = file.get('mimetype')

                url_private_download = file.get('url_private_download')
                new_file.download_url = SlackFilesClient.get_file_url(user_email, bot_name, bot_token,
                                                                      new_file.id, url_private_download,
                                                                      new_file.mimetype, cookies)
                result.files.append(new_file)

        js_ticks = int(result.id.split('.')[0] + result.id.split('.')[1][3:])
        result.created_at = datetime.fromtimestamp(js_ticks / 1000.0)
        return result


class SlackFilesClient:
    bucket_name = 'lgt_service_file'

    # Consider to cache these file somewhere in the super-fast cache solution
    @staticmethod
    def get_file_url(user_email, bot_name, bot_token, file_id, url_private_download, mimetype, cookies=None):
        storage_client = storage.Client()
        bucket = storage_client.get_bucket(SlackFilesClient.bucket_name)
        blob_path = f'slack_files/{user_email}/{bot_name}/{file_id}'
        blob = bucket.get_blob(blob_path)

        if not blob:
            res = requests.get(url_private_download, headers={'Authorization': f'Bearer {bot_token}'}, cookies=cookies)
            if res.status_code != 200:
                raise Exception(
                    f'Failed to download file: {url_private_download} from slack due to response: '
                    f'Code: {res.status_code} Error: {res.content}')
            blob = bucket.blob(blob_path)
            blob.upload_from_string(res.content, content_type=mimetype)

        blob = bucket.get_blob(blob_path)
        # valid for 3 days
        return blob.generate_signed_url(timedelta(3))


class SlackWebClient:
    def __init__(self, token, cookies=None):
        if isinstance(cookies, list):
            cookies = {c['name']: c['value'] for c in cookies}

        self.client = SlackClient(token, cookies)

    def delete_message(self, channel: str, id: str):
        return self.client.delete_message(channel, id)

    def update_message(self, channel: str, id: str, text: str, file_ids=''):
        return self.client.update_message(channel, id, text, file_ids)

    def get_profile(self, user_id):
        return self.client.user_info(user_id)

    def get_im_list(self):
        return self.client.get_im_list()

    def chat_history(self, channel):
        return self.client.conversations_history(channel)

    def post_message(self, to, text):
        return self.client.post_message(to, text)

    def user_list(self):
        return self.client.users_list()

    def channels_list(self):
        return self.client.get_conversations_list()

    def im_open(self, sender_id):
        return self.client.im_open(sender_id)

    def update_profile(self, profile):
        return self.client.update_profile(profile)

    def channel_join(self, channels):
        return self.client.join_channels(channels)

    def channel_leave(self, channels):
        return self.client.leave_channels(channels)

    def get_reactions(self, channel, id):
        return self.client.get_reactions(channel, id)

    def upload_file(self, file, file_name):
        return self.client.upload_file(file, file_name)

    def download_file(self, file_url):
        return self.client.download_file(file_url)

    def delete_file(self, file_id):
        return self.client.delete_file(file_id)

    def share_files(self, files_ids: list, channel: str, text: str = None) -> dict:
        return self.client.share_files(files_ids, channel, text)

    def check_email(self, email: str, user_agent: str) -> bool:
        return self.client.check_email(email, user_agent)

    def confirm_email(self, email: str, user_agent: str, locale: str = 'en-US') -> bool:
        return self.client.confirm_email(email, user_agent, locale)

    def confirm_code(self, email: str, code: str, user_agent: str, ) -> requests.Response:
        return self.client.confirm_code(email, code, user_agent)

    def find_workspaces(self, user_agent: str, ) -> requests.Response:
        return self.client.find_workspaces(user_agent)

    def conversation_replies(self, channel: str, id: str) -> dict:
        return self.client.conversations_replies(channel, id)

    def create_shared_invite(self):
        return self.client.create_shared_invite()

    def send_slack_invite_to_workspace(self, email: str):
        return self.client.send_slack_invite_to_workspace(email=email)

    def test_auth(self):
        return self.client.test_auth()

    def get_team_info(self):
        return self.client.get_team_info()



import os

project_id = os.environ.get('PROJECT_ID')
background_jobs_topic = os.environ.get('BACKGROUND_JOBS_TOPIC')
background_jobs_subscriber = os.environ.get('BACKGROUND_JOBS_SUBSCRIBER')
mongo_connection_string = os.environ.get('MONGO_CONNECTION_STRING')
smtp_host = os.environ.get('SMTP_HOST')
smtp_login = os.environ.get('SMTP_LOGIN')
smtp_password = os.environ.get('SMTP_PASSWORD')
google_app_credentials = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS')
portal_url = os.environ.get('PORTAL_URL')
service_account_email = os.environ.get('SERVICE_ACCOUNT_EMAIL')
service_account_password = os.environ.get('SERVICE_ACCOUNT_PASSWORD')
v2_server_host = os.environ.get('V2_SERVER_HOST')
v3_server_host = os.environ.get('V3_SERVER_HOST')
v1_server_host = os.environ.get('V1_SERVER_HOST')



from enum import Enum


class DiscordMethods(str, Enum):
    USER_GUILDS = 'users/@me/guilds'
    USER_DMS = 'users/@me/channels'
    USER = 'users/@me'
    LOGIN = 'auth/login'

    @staticmethod
    def guild_channels(guild_id: str):
        return f'guilds/{guild_id}/channels'

    @staticmethod
    def channels_invites(channel_id: str):
        return f'channels/{channel_id}/invites'



from lgt_jobs.lgt_data.model import UserModel
from lgt_jobs.lgt_data.mongo_repository import SubscriptionsRepository


def get_linkedin_search_contact(name: str):
    return f"https://www.linkedin.com/search/results/all/?keywords={name}&origin=GLOBAL_SEARCH_HEADER&sid=u%40F"


def get_help_text(user: UserModel):
    subscription = SubscriptionsRepository().find_one(id=user.subscription_id)
    return 'the onboarding call with us!' if subscription.trial else 'a call with us.'



from abc import ABC
import datetime
from typing import Optional, List
from pydantic import BaseModel, conlist
from pymongo import MongoClient
import logging as log
from lgt_jobs.basejobs import BaseBackgroundJob, BaseBackgroundJobData
from lgt_jobs.env import mongo_connection_string

"""
Track analytics
"""


class TrackAnalyticsJobData(BaseBackgroundJobData, BaseModel):
    data: str
    name: str
    event: str
    extra_ids: List[str] = []
    attributes: conlist(Optional[str])
    created_at: datetime.datetime


class TrackAnalyticsJob(BaseBackgroundJob, ABC):
    @property
    def job_data_type(self) -> type:
        return TrackAnalyticsJobData

    def exec(self, data: TrackAnalyticsJobData):
        with MongoClient(mongo_connection_string) as client:
            analytics_id = client.lgt_admin[data.event].insert_one(data.model_dump()).inserted_id
            log.info(f'[TrackAnalyticsJob]: Entry with id {analytics_id} has been recorded into {data.event}')
